/* signaltab.h --
 * Copyright 2012-13 Red Hat Inc.
 * All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Authors:
 *      Steve Grubb <sgrubb@redhat.com>
 * Location: include/uapi/asm-generic/signal.h
 */

_S(0,	"SIG0"		)
_S(1,	"SIGHUP"	)
_S(2,	"SIGINT"	)
_S(3,	"SIGQUIT"	)
_S(4,	"SIGILL"	)
_S(5,	"SIGTRAP"	)
_S(6,	"SIGABRT"	)
_S(7,	"SIGBUS"	)
_S(8,	"SIGFPE"	)
_S(9,	"SIGKILL"	)
_S(10,	"SIGUSR1"	)
_S(11,	"SIGSEGV"	)
_S(12,	"SIGUSR2"	)
_S(13,	"SIGPIPE"	)
_S(14,	"SIGALRM"	)
_S(15,	"SIGTERM"	)
_S(16,	"SIGSTKFLT"	)
_S(17,	"SIGCHLD"	)
_S(18,	"SIGCONT"	)
_S(19,	"SIGSTOP"	)
_S(20,	"SIGTSTP"	)
_S(21,	"SIGTTIN"	)
_S(22,	"SIGTTOU"	)
_S(23,	"SIGURG"	)
_S(24,	"SIGXCPU"	)
_S(25,	"SIGXFSZ"	)
_S(26,	"SIGVTALRM"	)
_S(27,	"SIGPROF"	)
_S(28,	"SIGWINCH"	)
_S(29,	"SIGIO"		)
_S(30,	"IGPWR"		)
_S(31,	"SIGSYS"	)

