/* dso.h -- 
 * Copyright 2005,2006,2009 Red Hat Inc., Durham, North Carolina.
 * All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Authors:
 *	Steve Grubb <sgrubb@redhat.com>
 */
#ifndef _DSO_H_
#define _DSO_H_

/*
 * This is to mark functions as internal to the API
 */
#ifndef AUDIT_HIDDEN_START
#define AUDIT_HIDDEN_START _Pragma("GCC visibility push(hidden)")
#endif

/*
 * This ends the section that is internal to the API.
 */
#ifndef AUDIT_HIDDEN_END
#define AUDIT_HIDDEN_END _Pragma("GCC visibility pop")
#endif

#endif

