/* icmptypetab.h --
 * Copyright 2011-13 Red Hat Inc.
 * All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Authors:
 *      Steve Grubb <sgrubb@redhat.com>
 * Location: include/uapi/linux/icmp.h
 */

_S(0,	"echo-reply"			)
_S(3,	"destination-unreachable"	)
_S(4,	"source-quench"			)
_S(5,	"redirect"			)
_S(8,	"echo"				)
_S(11,	"time-exceeded"			)
_S(12,	"parameter-problem"		)
_S(13,	"timestamp-request"		)
_S(14,	"timestamp-reply"		)
_S(15,	"info-request"			)
_S(16,	"info-reply"			)
_S(17,	"address-mask-request"		)
_S(18,	"address-mask-reply"		)

