/* normalize_evtypetab.h --
 * Copyright 2017,2021 Red Hat Inc.
 * All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Authors:
 *      Steve Grubb <sgrubb@redhat.com>
 */

#include "normalize-internal.h"

_S(NORM_EVTYPE_UNKNOWN,		"unknown"	)
_S(NORM_EVTYPE_USERSPACE,	"user-space"	)
_S(NORM_EVTYPE_SYSTEM_SERVICES,	"system-services" )
_S(NORM_EVTYPE_CONFIG,		"configuration"	)
_S(NORM_EVTYPE_TTY,		"TTY"		)
_S(NORM_EVTYPE_USER_ACCT,	"user-account"	)
_S(NORM_EVTYPE_USER_LOGIN,	"user-login"	)
_S(NORM_EVTYPE_AUDIT_DAEMON,	"audit-daemon"	)
_S(NORM_EVTYPE_MAC_DECISION,	"mac-decision"	)
_S(NORM_EVTYPE_ANOMALY,		"anomaly"	)
_S(NORM_EVTYPE_INTEGRITY,	"integrity"	)
_S(NORM_EVTYPE_ANOMALY_RESP,	"anomaly-response")
_S(NORM_EVTYPE_MAC,		"mac"		)
_S(NORM_EVTYPE_CRYPTO,		"crypto"	)
_S(NORM_EVTYPE_VIRT,		"virt"		)
_S(NORM_EVTYPE_AUDIT_RULE,	"audit-rule"	)
_S(NORM_EVTYPE_DAC_DECISION,	"dac-decision"	)
_S(NORM_EVTYPE_GROUP_CHANGE,	"group-change"	)
_S(NORM_EVTYPE_AV_DECISION,	"av-decision"	)
_S(NORM_EVTYPE_BPF,		"bpf-program"   )
