/* pktoptnametab.h --
 * Copyright 2013-14,2020 Red Hat Inc.
 * All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Authors:
 *      Steve Grubb <sgrubb@redhat.com>
 * Location: include/uapi/linux/if_packet.h
 */

_S(1, "PACKET_ADD_MEMBERSHIP")
_S(2, "PACKET_DROP_MEMBERSHIP")
_S(3, "PACKET_RECV_OUTPUT")
_S(5, "PACKET_RX_RING")
_S(6, "PACKET_STATISTICS")
_S(7, "PACKET_COPY_THRESH")
_S(8, "PACKET_AUXDATA")
_S(9, "PACKET_ORIGDEV")
_S(10, "PACKET_VERSION")
_S(11, "PACKET_HDRLEN")
_S(12, "PACKET_RESERVE")
_S(13, "PACKET_TX_RING")
_S(14, "PACKET_LOSS")
_S(15, "PACKET_VNET_HDR")
_S(16, "PACKET_TX_TIMESTAMP")
_S(17, "PACKET_TIMESTAMP")
_S(18, "PACKET_FANOUT")
_S(19, "PACKET_TX_HAS_OFF")
_S(20, "PACKET_QDISC_BYPASS")
_S(21, "PACKET_ROLLOVER_STATS")
_S(22, "PACKET_FANOUT_DATA")
_S(23, "PACKET_IGNORE_OUTGOING")
_S(24, "PACKET_VNET_HDR_SZ")

