/* ip6optnametab.h --
 * Copyright 2013-16,2018,2020 Red Hat Inc.
 * All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Authors:
 *      Steve Grubb <sgrubb@redhat.com>
 * Location: include/uapi/linux/in6.h (all ^IPV6_ && ^MCAST_)
 *           include/uapi/linux/netfilter_ipv6/ip6_tables.h (all ^IP6T_)
 *           include/uapi/linux/mroute6.h  (not used?)
 */

_S(1, "IPV6_ADDRFORM")
_S(2, "IPV6_2292PKTINFO")
_S(3, "IPV6_2292HOPOPTS")
_S(4, "IPV6_2292DSTOPTS")
_S(5, "IPV6_2292RTHDR")
_S(6, "IPV6_2292PKTOPTIONS")
_S(7, "IPV6_CHECKSUM")
_S(8, "IPV6_2292HOPLIMIT")
_S(9, "IPV6_NEXTHOP")
_S(10, "IPV6_AUTHHDR")
_S(11, "IPV6_FLOWINFO")
_S(16, "IPV6_UNICAST_HOPS")
_S(17, "IPV6_MULTICAST_IF")
_S(18, "IPV6_MULTICAST_HOPS")
_S(19, "IPV6_MULTICAST_LOOP")
_S(20, "IPV6_ADD_MEMBERSHIP")
_S(21, "IPV6_DROP_MEMBERSHIP")
_S(22, "IPV6_ROUTER_ALERT")
_S(23, "IPV6_MTU_DISCOVER")
_S(24, "IPV6_MTU")
_S(25, "IPV6_RECVERR")
_S(26, "IPV6_V6ONLY")
_S(27, "IPV6_JOIN_ANYCAST")
_S(28, "IPV6_LEAVE_ANYCAST")
_S(29, "IPV6_MULTICAST_ALL")
_S(30, "IPV6_ROUTER_ALERT_ISOLATE")
_S(31, "IPV6_RECVERR_RFC4884")
_S(32, "IPV6_FLOWLABEL_MGR")
_S(33, "IPV6_FLOWINFO_SEND")
_S(34, "IPV6_IPSEC_POLICY")
_S(35, "IPV6_XFRM_POLICY")
_S(36, "IPV6_HDRINCL")
_S(42, "MCAST_JOIN_GROUP")
_S(43, "MCAST_BLOCK_SOURCE")
_S(44, "MCAST_UNBLOCK_SOURCE")
_S(45, "MCAST_LEAVE_GROUP")
_S(46, "MCAST_JOIN_SOURCE_GROUP")
_S(47, "MCAST_LEAVE_SOURCE_GROUP")
_S(48, "MCAST_MSFILTER")
_S(49, "IPV6_RECVPKTINFO")
_S(50, "IPV6_PKTINFO")
_S(51, "IPV6_RECVHOPLIMIT")
_S(52, "IPV6_HOPLIMIT")
_S(53, "IPV6_RECVHOPOPTS")
_S(54, "IPV6_HOPOPTS")
_S(55, "IPV6_RTHDRDSTOPTS")
_S(56, "IPV6_RECVRTHDR")
_S(57, "IPV6_RTHDR")
_S(58, "IPV6_RECVDSTOPTS")
_S(59, "IPV6_DSTOPTS")
_S(60, "IPV6_RECVPATHMTU")
_S(61, "IPV6_PATHMTU")
_S(62, "IPV6_DONTFRAG")
_S(63, "IPV6_USE_MIN_MTU")
_S(64, "IP6T_SO_SET_REPLACE")
_S(65, "IP6T_SO_SET_ADD_COUNTERS")
_S(66, "IPV6_RECVTCLASS")
_S(67, "IPV6_TCLASS")
_S(68, "IP6T_SO_GET_REVISION_MATCH")
_S(69, "IP6T_SO_GET_REVISION_TARGET")
_S(70, "IPV6_AUTOFLOWLABEL")
_S(72, "IPV6_ADDR_PREFERENCES")
_S(73, "IPV6_MINHOPCOUNT")
_S(74, "IPV6_ORIGDSTADDR")
_S(75, "IPV6_TRANSPARENT")
_S(76, "IPV6_UNICAST_IF")
_S(77, "IPV6_RECVFRAGSIZE")
_S(78, "IPV6_FREEBIND")
_S(80, "IP6T_SO_ORIGINAL_DST")

