/* captab.h --
 * Copyright 2007,2008,2012-14,2021 Red Hat Inc.
 * All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Authors:
 *      Steve Grubb <sgrubb@redhat.com>
 * Location: include/uapi/linux/capability.h
 */


_S(0,	"chown"			)
_S(1,	"dac_override"		)
_S(2,	"dac_read_search"	)
_S(3,	"fowner"		)
_S(4,	"fsetid"		)
_S(5,	"kill"			)
_S(6,	"setgid"		)
_S(7,	"setuid"		)
_S(8,	"setpcap"		)
_S(9,	"linux_immutable"	)
_S(10,	"net_bind_service"	)
_S(11,	"net_broadcast"		)
_S(12,	"net_admin"		)
_S(13,	"net_raw"		)
_S(14,	"ipc_lock"		)
_S(15,	"ipc_owner"		)
_S(16,	"sys_module"		)
_S(17,	"sys_rawio"		)
_S(18,	"sys_chroot"		)
_S(19,	"sys_ptrace"		)
_S(20,	"sys_pacct"		)
_S(21,	"sys_admin"		)
_S(22,	"sys_boot"		)
_S(23,	"sys_nice"		)
_S(24,	"sys_resource"		)
_S(25,	"sys_time"		)
_S(26,	"sys_tty_config"	)
_S(27,	"mknod"			)
_S(28,	"lease"			)
_S(29,	"audit_write"		)
_S(30,	"audit_control"		)
_S(31,	"setfcap"		)
_S(32,	"mac_override"		)
_S(33,	"mac_admin"		)
_S(34,	"syslog"		)
_S(35,	"wake_alarm"		)
_S(36,	"block_suspend"		)
_S(37,	"audit_read"		)
_S(38,  "perfmon"               )
_S(39,  "bpf"                   )
_S(40,  "checkpoint_restore"    )

