/* fsconfig.h
 * Copyright 2023 Red Hat Inc.
 * All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor
 * Boston, MA 02110-1335, USA.
 *
 * Authors:
 *      Steve Grubb <sgrubb@redhat.com>
 * Location: include/uapi/linux/mount.h
 */

_S(0,   "FSCONFIG_SET_FLAG")
_S(1,   "FSCONFIG_SET_STRING")
_S(2,   "FSCONFIG_SET_BINARY")
_S(3,   "FSCONFIG_SET_PATH")
_S(4,   "FSCONFIG_SET_PATH_EMPTY")
_S(5,   "FSCONFIG_SET_FD")
_S(6,   "FSCONFIG_CMD_CREATE")
_S(7,   "FSCONFIG_CMD_RECONFIGURE")
_S(8,   "FSCONFIG_CMD_CREATE_EXCL")

