/* tcpoptnametab.h --
 * Copyright 2013-14,2018 Red Hat Inc.
 * All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1335, USA
 *
 * Authors:
 *      Steve Grubb <sgrubb@redhat.com>
 * Location: include/uapi/linux/tcp.h
 */

_S(1, "TCP_NODELAY")
_S(2, "TCP_MAXSEG")
_S(3, "TCP_CORK")
_S(4, "TCP_KEEPIDLE")
_S(5, "TCP_KEEPINTVL")
_S(6, "TCP_KEEPCNT")
_S(7, "TCP_SYNCNT")
_S(8, "TCP_LINGER2")
_S(9, "TCP_DEFER_ACCEPT")
_S(10, "TCP_WINDOW_CLAMP")
_S(11, "TCP_INFO")
_S(12, "TCP_QUICKACK")
_S(13, "TCP_CONGESTION")
_S(14, "TCP_MD5SIG")
_S(15, "TCP_COOKIE_TRANSACTIONS")
_S(16, "TCP_THIN_LINEAR_TIMEOUTS")
_S(17, "TCP_THIN_DUPACK")
_S(18, "TCP_USER_TIMEOUT")
_S(19, "TCP_REPAIR")
_S(20, "TCP_REPAIR_QUEUE")
_S(21, "TCP_QUEUE_SEQ")
_S(22, "TCP_REPAIR_OPTIONS")
_S(23, "TCP_FASTOPEN")
_S(24, "TCP_TIMESTAMP")
_S(25, "TCP_NOTSENT_LOWAT")
_S(26, "TCP_CC_INFO")
_S(27, "TCP_SAVE_SYN")
_S(28, "TCP_SAVED_SYN")
_S(29, "TCP_REPAIR_WINDOW")
_S(30, "TCP_FASTOPEN_CONNECT")
_S(31, "TCP_ULP")
_S(32, "TCP_MD5SIG_EXT")
_S(33, "TCP_FASTOPEN_KEY")
_S(34, "TCP_FASTOPEN_NO_COOKIE")
_S(35, "TCP_ZEROCOPY_RECEIVE")
_S(36, "TCP_INQ")
_S(37, "TCP_TX_DELAY")
_S(38, "TCP_AO_ADD_KEY")
_S(39, "TCP_AO_DEL_KEY")
_S(40, "TCP_AO_INFO")
_S(41, "TCP_AO_GET_KEYS")
_S(42, "TCP_AO_REPAIR")

