/*
 * normalize_obj_kind_map.h
 * Copyright (c) 2016-24 Red Hat Inc.
 * All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Authors:
 *   Steve Grubb <sgrubb@redhat.com>
 */
#include "normalize-internal.h"

_S(NORM_WHAT_UNKNOWN, "unknown")
_S(NORM_WHAT_FIFO, "fifo")
_S(NORM_WHAT_CHAR_DEV, "character-device")
_S(NORM_WHAT_DIRECTORY, "directory")
_S(NORM_WHAT_BLOCK_DEV, "block-device")
_S(NORM_WHAT_FILE, "file")
_S(NORM_WHAT_FILESYSTEM, "file-system")
_S(NORM_WHAT_LINK, "symlink")
_S(NORM_WHAT_SOCKET, "socket")
_S(NORM_WHAT_PROCESS, "process")
_S(NORM_WHAT_FIREWALL, "firewall")
_S(NORM_WHAT_SERVICE, "service")
_S(NORM_WHAT_ACCT, "account")
_S(NORM_WHAT_USER_SESSION, "user-session")
_S(NORM_WHAT_VM, "virtual-machine")
_S(NORM_WHAT_PRINTER, "printer")
_S(NORM_WHAT_SYSTEM, "system")
_S(NORM_WHAT_AUDIT_RULE, "admin-defined-rule")
_S(NORM_WHAT_AUDIT_CONFIG, "audit-config")
_S(NORM_WHAT_SECURITY_POLICY, "security-policy")
_S(NORM_WHAT_MEMORY, "memory")
_S(NORM_WHAT_KEYSTROKES, "keystrokes")
_S(NORM_WHAT_DEVICE, "device")
_S(NORM_WHAT_SOFTWARE, "software")
_S(NORM_WHAT_INTEGRITY_POLICY, "integrity-policy")
_S(NORM_WHAT_SECURITY_MODULES, "security-modules")
_S(NORM_WHAT_PROCESS_GROUP, "process-group")
//_S(, "")
