/* sockleveltab.h --
 * Copyright 2013-15,2018 Red Hat Inc.
 * All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1335, USA
 *
 * Authors:
 *      Steve Grubb <sgrubb@redhat.com>
 * Location: include/linux/socket.h
 */


_S(0, "SOL_IP")
_S(6, "SOL_TCP")
_S(17, "SOL_UDP")
_S(41, "SOL_IPV6")
_S(58, "SOL_ICMPV6")
_S(132, "SOL_SCTP")
_S(136, "SOL_UDPLITE")
_S(255, "SOL_RAW")
_S(256, "SOL_IPX")
_S(257, "SOL_AX25")
_S(258, "SOL_ATALK")
_S(259, "SOL_NETROM")
_S(260, "SOL_ROSE")
_S(261, "SOL_DECNET")
_S(263, "SOL_PACKET")
_S(264, "SOL_ATM")
_S(265, "SOL_AAL")
_S(266, "SOL_IRDA")
_S(267, "SOL_NETBEUI")
_S(268, "SOL_LLC")
_S(269, "SOL_DCCP")
_S(270, "SOL_NETLINK")
_S(271, "SOL_TIPC")
_S(272, "SOL_RXRPC")
_S(273, "SOL_PPPOL2TP")
_S(274, "SOL_BLUETOOTH")
_S(275, "SOL_PNPIPE")
_S(276, "SOL_RDS")
_S(277, "SOL_IUCV")
_S(278, "SOL_CAIF")
_S(279, "SOL_ALG")
_S(280, "SOL_NFC")
_S(281, "SOL_KCM")
_S(282, "SOL_TLS")
_S(283, "SOL_XDP")
_S(284, "SOL_MPTCP")
_S(285, "SOL_MCTP")
_S(286, "SOL_SMC")
_S(287, "SOL_VSOCK")

