/* umounttab.h --
 * Copyright 2013 Red Hat Inc.
 * All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Authors:
 *      Steve Grubb <sgrubb@redhat.com>
 * Location: include/linux/fs.h
 */


_S(0x00000001,	"MNT_FORCE"		)
_S(0x00000002,	"MNT_DETACH"		)
_S(0x00000004,	"MNT_EXPIRE"		)
_S(0x00000008,	"UMOUNT_NOFOLLOW"	)
_S(0x80000000,	"UMOUNT_UNUSED"		)

