/* clocktab.h --
 * Copyright 2012,2014 Red Hat Inc.
 * All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Authors:
 *      Steve Grubb <sgrubb@redhat.com>
 * Location: include/uapi/linux/time.h
 */

_S(0,	"CLOCK_REALTIME"		)
_S(1,	"CLOCK_MONOTONIC"		)
_S(2,	"CLOCK_PROCESS_CPUTIME_ID"	)
_S(3,	"CLOCK_THREAD_CPUTIME_ID"	)
_S(4,	"CLOCK_MONOTONIC_RAW"		)
_S(5,	"CLOCK_REALTIME_COARSE"		)
_S(6,	"CLOCK_MONOTONIC_COARSE"	)
_S(7,	"CLOCK_BOOTTIME"		)
_S(8,	"CLOCK_REALTIME_ALARM"		)
_S(9,	"CLOCK_BOOTTIME_ALARM"		)
_S(10,	"CLOCK_SGI_CYCLE"		)
_S(11,	"CLOCK_TAI"			)

