/* rlimittab.h --
 * Copyright 2012-13 Red Hat Inc.
 * All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Authors:
 *      Steve Grubb <sgrubb@redhat.com>
 * Location: include/uapi/asm-generic/resource.h
 */

 _S(0, "RLIMIT_CPU")
 _S(1, "RLIMIT_FSIZE")
 _S(2, "RLIMIT_DATA")
 _S(3, "RLIMIT_STACK")
 _S(4, "RLIMIT_CORE")
 _S(5, "RLIMIT_RSS")
 _S(6, "RLIMIT_NPROC")
 _S(7, "RLIMIT_NOFILE")
 _S(8, "RLIMIT_MEMLOCK")
 _S(9, "RLIMIT_AS")
 _S(10,"RLIMIT_LOCKS")
 _S(11,"RLIMIT_SIGPENDING")
 _S(12,"RLIMIT_MSGQUEUE")
 _S(13,"RLIMIT_NICE")
 _S(14,"RLIMIT_RTPRIO")
 _S(15,"RLIMIT_RTTIME")

